#include "decodeur.h"
#include <cmath>

DecodeurNMEA2000::DecodeurNMEA2000()
{
}

DecodeurNMEA2000::~DecodeurNMEA2000()
{
}

double DecodeurNMEA2000::extraireTemperature(const BYTE DATA[8])
{
   int dataTemperature = 0;
   double temperature = 0.;
   
   // Attention : little endian ! sinon cf. doc
   dataTemperature = ( ((DATA[3] & 0x00FF) << 8) | (DATA[2] & 0x00FF)) & 0x0000FFFF;
   
   #ifdef DEBUG_STATION
   printf("message.DATA[2] = 0x%02x\n", DATA[2]);
   printf("message.DATA[3] = 0x%02x\n", DATA[3]);
   #endif
   
   // Temperature : cf. doc
   temperature = ( ( ((double)dataTemperature*0.01) ) - 273.15 );
   
   #ifdef DEBUG_STATION
   printf("dataTemperature = %d\n", dataTemperature);
   printf("temperature = %.2f °C\n", temperature);
   #endif

   return temperature;
}

double DecodeurNMEA2000::extrairePressionAtmospherique(const BYTE DATA[8])
{
   int dataPressionAtmospherique = 0;
   double pressionAtmospherique = 0.;
   
   // Attention : little endian ! sinon cf. doc
   dataPressionAtmospherique = ( ((DATA[7] & 0x00FF) << 8) | (DATA[6] & 0x00FF)) & 0x0000FFFF;
   
   #ifdef DEBUG_STATION
   printf("message.DATA[6] = 0x%02x\n", DATA[6]);
   printf("message.DATA[7] = 0x%02x\n", DATA[7]);
   #endif
   
   // Pression atmospherique : cf. doc
   pressionAtmospherique = ( ((double)dataPressionAtmospherique) );
   
   #ifdef DEBUG_STATION
   printf("dataPressionAtmospherique = %d\n", dataPressionAtmospherique);
   printf("pressionAtmospherique = %.2f °C\n", pressionAtmospherique);
   #endif

   return pressionAtmospherique;
}

double DecodeurNMEA2000::extraireHygrometrie(const BYTE DATA[8])
{
   int dataHygrometrie = 0;
   double hygrometrie = 0.;

   // Attention : little endian ! sinon cf. doc
   dataHygrometrie = ( ((DATA[5] & 0x00FF) << 8) | (DATA[4] & 0x00FF)) & 0x0000FFFF;
   
   #ifdef DEBUG_STATION
   printf("message.DATA[4] = 0x%02x\n", DATA[4]);
   printf("message.DATA[5] = 0x%02x\n", DATA[5]);
   #endif
   
   // Hygrometrie : cf. doc
   hygrometrie = ( ((double)dataHygrometrie) );
   
   #ifdef DEBUG_STATION
   printf("dataHygrometrie = %d\n", dataHygrometrie);
   printf("hygrometrie = %.2f °C\n", hygrometrie);
   #endif

   return hygrometrie;
}

double DecodeurNMEA2000::extraireVitesseVent(const BYTE DATA[8])
{
   int dataVitesseVent = 0;
   double vitesseVent = 0.;
   
   // Attention : little endian ! sinon cf. doc
   dataVitesseVent = ( ((DATA[2] & 0x00FF) << 8) | (DATA[1] & 0x00FF)) & 0x0000FFFF; // mm/s
   
   #ifdef DEBUG_STATION
   printf("message.DATA[1] = 0x%02x\n", DATA[1]);
   printf("message.DATA[2] = 0x%02x\n", DATA[2]);
   #endif
   
   // Vitesse du vent : cf. doc
   vitesseVent  = ( ((((double)dataVitesseVent * 10.0)/(1000.0))/1000.0) );
   
   #ifdef DEBUG_STATION
   printf("dataVitesseVent = %d\n", dataVitesseVent);
   printf("vitesseVent = %.2f °C\n", vitesseVent);
   #endif

   return vitesseVent;
}

double DecodeurNMEA2000::extraireDirectionVent(const BYTE DATA[8])
{
   int dataDirectionVent = 0;
   double directionVent = 0.;
   
   // Attention : little endian ! sinon cf. doc
   dataDirectionVent = ( ((DATA[4] & 0x00FF) << 8) | (DATA[3] & 0x00FF)) & 0x0000FFFF; //rad
   
   #ifdef DEBUG_STATION
   printf("message.DATA[3] = 0x%02x\n", DATA[3]);
   printf("message.DATA[4] = 0x%02x\n", DATA[4]);
   #endif
   
   // Direction du vent : cf. doc
   directionVent = ( ((double)dataDirectionVent * 0.0001)*360/(M_PI) );
   
   #ifdef DEBUG_STATION
   printf("dataDirectionVent = %d\n", dataDirectionVent);
   printf("directionVent = %.2f °C\n", directionVent);
   #endif

   return directionVent;
}
