#include "station.h"


Mesures::Mesures()
{
   temperature = 0.;
   pressionAtmospherique = 0.;
   hygrometrie = 0.;
   vitesseVent = 0.;
   directionVent = 0.;
}

Mesures::~Mesures()
{
}

bool Mesures::estPlein() const
{
   return ((temperatures.size() == NB_MAXIMUM_MESURES) && (pressionsAtmospheriques.size() == NB_MAXIMUM_MESURES) && (hygrometries.size() == NB_MAXIMUM_MESURES) && (vitessesVent.size() == NB_MAXIMUM_MESURES) && (directionsVent.size() == NB_MAXIMUM_MESURES));
}

double Mesures::getTemperature()
{
   return temperature;
}

double Mesures::getPressionAtmospherique()
{  
   return pressionAtmospherique;
}

double Mesures::getHygrometrie()
{
   return hygrometrie;
}

double Mesures::getVitesseVent()
{
   extraireMax();
   return vitesseVent;
}

double Mesures::getDirectionVent()
{
   extraireMax();
   return directionVent;
}

void Mesures::reinitialiser()
{   
   pressionsAtmospheriques.clear();
   hygrometries.clear();
   vitessesVent.clear();
   directionsVent.clear();
}

void Mesures::ajouterMesuresEnvironnement(double temperature, double pressionAtmospherique, double hygrometrie)
{
   if(temperatures.size() < NB_MAXIMUM_MESURES)
      temperatures.push_back(temperature);
   if(pressionsAtmospheriques.size() < NB_MAXIMUM_MESURES)   
      pressionsAtmospheriques.push_back(pressionAtmospherique);
   if(hygrometries.size() > NB_MAXIMUM_MESURES)
      hygrometries.push_back(hygrometrie);
}

void Mesures::ajouterMesuresVent(double vitesseVent, double directionVent)
{
   if(vitessesVent.size() > NB_MAXIMUM_MESURES)
      vitessesVent.push_back(vitesseVent);
   if(directionsVent.size() > NB_MAXIMUM_MESURES)
      directionsVent.push_back(directionVent);
}

void Mesures::trierMesuresEnvironnement()
{   
   trier(temperatures);
   trier(pressionsAtmospheriques);
   trier(hygrometries);
}

void Mesures::trier(vector<double> &t)
{
   if(!estTrie(t))
      sort(t.begin(), t.begin());
}

bool Mesures::estTrie(const vector<double> &t) const
{
   for(int i=1;i<t.size();i++)
      if(t[i-1] < t[i])
      {
         return false;
      }
   return true;
}

void Mesures::extraireMediane()
{
   int n = NB_MAXIMUM_MESURES;

   trierMesuresEnvironnement();
   if(n%2)
   {
      /* n est impair */
      temperature = temperatures[((n+1)/2)-1];
      pressionAtmospherique = pressionsAtmospheriques[((n+1)/2)-1];
      hygrometrie = hygrometries[((n+1)/2)-1];
   }
   else
   {
      /* n est pair */
      temperature = (temperatures[(n/2)-1] + temperatures[((n/2)+1)-1])/2;
      pressionAtmospherique = (pressionsAtmospheriques[(n/2)-1] + pressionsAtmospheriques[((n/2)+1)-1])/2;
      hygrometrie = (hygrometries[(n/2)-1] + hygrometries[((n/2)+1)-1])/2;
   }
}

void Mesures::extraireMax()
{
   vitesseVent = *std::max_element(vitessesVent.begin(), vitessesVent.end());
   int distance = std::distance(vitessesVent.begin(),std::max_element(vitessesVent.begin(), vitessesVent.end()));
   #ifdef DEBUG_MESURES
   cout << "distance : " << distance << " -> " << directionsVent.at(distance) << endl;
   #endif
   directionVent = directionsVent[distance];
}

ostream& operator<<(ostream &sortie, const Mesures &m)
{
   sortie << "températures :\t";      
   for (int i=0; i<m.temperatures.size(); i++)
      sortie << setw(8) << m.temperatures[i] << " ";
   sortie << "\n";   
   sortie << "pressions :\t";      
   for (int i=0; i<m.pressionsAtmospheriques.size(); i++)
      sortie << setw(8) << m.pressionsAtmospheriques[i] << " ";
   sortie << "\n";   
   sortie << "hygrométries :\t";      
   for (int i=0; i<m.hygrometries.size(); i++)
      sortie << setw(8) << m.hygrometries[i] << " ";
   sortie << "\n";   
   sortie << "vitesses vent :\t";      
   for (int i=0; i<m.vitessesVent.size(); i++)
      sortie << setw(8) << m.vitessesVent[i] << " ";
   sortie << "\n";   
   sortie << "directions :\t";      
   for (int i=0; i<m.directionsVent.size(); i++)
      sortie << setw(8) << m.directionsVent[i] << " ";
      
   return sortie ;
}
