#include "station.h"
#include "decodeur.h"

StationMeteo::StationMeteo()
{
   this->lieu = "";
   interfacePeakNMEA2000.initialiser();
   temperature = 0.;
   pressionAtmospherique = 0.;
   hygrometrie = 0.;
   vitesseVent = 0.;
   directionVent = 0.;
   timestamp = -1;
   ofile.open ("st.csv");
   ofile << "Station Météo à " << getLieu() << endl << endl;
   ofile << "date;heure;température;pression atmosphérique;hygrométrie relative;vitesse du vent;direction du vent;\n";

   setlocale(LC_ALL, "");
}

StationMeteo::StationMeteo(std::string lieu)
{
   this->lieu = lieu;
   interfacePeakNMEA2000.initialiser();
   temperature = 0.;
   pressionAtmospherique = 0.;
   hygrometrie = 0.;
   vitesseVent = 0.;
   directionVent = 0.;
   timestamp = -1;
   ofile.open ("st.csv");
   ofile << "Station Météo à " << getLieu() << endl << endl;
   ofile << "date;heure;température;pression atmosphérique;hygrométrie relative;vitesse du vent;direction du vent;\n";
   setlocale(LC_ALL, "");
}

StationMeteo::~StationMeteo()
{
   //interfacePeakNMEA2000.afficherInfos();
   interfacePeakNMEA2000.fermer();
   ofile.close();
}

std::string StationMeteo::getLieu() const
{
   return "";
}

double StationMeteo::getTemperature()
{
   
   
   return temperature;
}

double StationMeteo::getPressionAtmospherique()
{
   
   
   return pressionAtmospherique;
}

double StationMeteo::getHygrometrie()
{
   
   
   return hygrometrie;
}

double StationMeteo::getVitesseVent()
{
   
   
   return vitesseVent;
}

double StationMeteo::getDirectionVent()
{
   
   
   return directionVent;
}

time_t StationMeteo::getTimestamp()
{
   return timestamp;
}

string StationMeteo::getHorodatage()
{
   string horodatage = "";
   time_t ts = getTimestamp();
   struct tm *tmp;
   char date[128];

   tmp = localtime(&ts);
   if (tmp == NULL) 
   {
      return horodatage;
   }
   /*if (strftime(date, sizeof(date), "%c", tmp) == 0) 
   {
      return horodatage;
   }*/
   if (strftime(date, sizeof(date), "%j/%m/%Y à %t", tmp) == 0) 
   {
      return horodatage;
   }
   horodatage = date;
   
   return horodatage;
}

string StationMeteo::getDate()
{
   string horodatage = "";
   time_t ts = getTimestamp();
   struct tm *tmp;
   char date[128];

   tmp = localtime(&ts);
   if (tmp == NULL) 
   {
      return horodatage;
   }
   if (strftime(date, sizeof(date), "%j/%m/%Y", tmp) == 0) 
   {
      return horodatage;
   }
   horodatage = date;
   
   return horodatage;
}

string StationMeteo::getHeure()
{
   string horodatage = "";
   time_t ts = getTimestamp();
   struct tm *tmp;
   char date[128];

   tmp = localtime(&ts);
   if (tmp == NULL) 
   {
      return horodatage;
   }   
   if (strftime(date, sizeof(date), "%t", tmp) == 0) 
   {
      return horodatage;
   }
   horodatage = date;
   
   return horodatage;
}

void StationMeteo::acquerir()
{
   int pgn;
   TPCANMsg message;
   
   mesures.reinitialiser();
   do
   {
      // les informations de temperature, hygrometrie et pression atmosphérique se trouvent dans un message NMEA2000 d'identifiant PGN 130311
      pgn = ENVIRONMENTAL_PARAMETERS; // = 130311   
      message = acquerir(pgn);   
      decoder(pgn, message.DATA);
      ajouterMesures(pgn);
   
      // les informations sur le vent se trouvent dans un message NMEA2000 d'identifiant PGN 130306
      pgn = WIND_DATA; // = 130306
      message = acquerir(pgn);   
      decoder(pgn, message.DATA);
      ajouterMesures(pgn);
   }
   while(!mesures.estPlein());
   
   #ifdef DEBUG_STATION
   cout << mesures << endl;
   #endif
   
   temperature = mesures.getTemperature();
   pressionAtmospherique = mesures.getPressionAtmospherique();
   hygrometrie = mesures.getHygrometrie();
   vitesseVent = mesures.getVitesseVent();
   directionVent = mesures.getDirectionVent();
   
   #ifdef DEBUG_STATION
   cout << mesures << endl;
   #endif
   
   // horodatage
   horodater();   
}

void StationMeteo::enregistrer(bool acquisition/*=false*/)
{
   if(!ofile.is_open())
      return;
      
   if(acquisition == true)
      acquerir();
   
   //"date;heure;température;pression atmosphérique;hygrométrie relative;vitesse du vent;direction du vent;\n";
   ofile << getDate() << ";" << getHeure() << ";" << getTemperature() << ";" << getPressionAtmospherique() << ";" << getHygrometrie() << ";" << getVitesseVent() << ";" << getDirectionVent() << ";" << endl;
}

TPCANMsg StationMeteo::acquerir(int pgn)
{
   TPCANMsg message;
   
   interfacePeakNMEA2000.obtenirMessage(pgn, message);
   
   return message;
}

bool StationMeteo::decoder(int pgn, const BYTE DATA[8])
{       
   switch(pgn)
   {
      case ENVIRONMENTAL_PARAMETERS :
         #ifdef DEBUG_STATION
         printf("PGN : %d\n", pgn);
         #endif         
         temperature = DecodeurNMEA2000::extraireTemperature(DATA);
         pressionAtmospherique = DecodeurNMEA2000::extrairePressionAtmospherique(DATA);
         hygrometrie = DecodeurNMEA2000::extraireHygrometrie(DATA);
         break;
      case WIND_DATA :
         vitesseVent = DecodeurNMEA2000::extraireVitesseVent(DATA);
         directionVent = DecodeurNMEA2000::extraireDirectionVent(DATA);
         break;   
   }
   return false;
}

void StationMeteo::horodater()
{
   timestamp = time(NULL);
}

void StationMeteo::ajouterMesures(int pgn)
{       
   switch(pgn)
   {
      case ENVIRONMENTAL_PARAMETERS :
         #ifdef DEBUG_STATION
         printf("PGN : %d\n", pgn);
         #endif         
         mesures.ajouterMesuresEnvironnement(temperature, pressionAtmospherique, hygrometrie);
         break;
      case WIND_DATA :
         break;   
   }
}
