#ifndef STATION_H
#define STATION_H

#include "interface.h"
#include "mesures.h"
#include <string>
#include <fstream> /* pour ofstream */
#include <clocale>

// PGN des messages NMEA2000 de la station WSO100 (cf. doc)
#define ENVIRONMENTAL_PARAMETERS 130311 // (2 trames / s)
#define WIND_DATA                130306 // (10 trames / s)

#define DEBUG_STATION

class StationMeteo
{
   public:
      StationMeteo();
      StationMeteo(std::string lieu);
      ~StationMeteo();
       
      std::string getLieu() const; 
      double getTemperature();
      double getPressionAtmospherique();
      double getHygrometrie();
      double getVitesseVent();
      double getDirectionVent();
      time_t getTimestamp();
      string getHorodatage();
      string getDate();
      string getHeure();
      void   acquerir();
      void   enregistrer(bool acquisition=false);
      
      TPCANMsg acquerir(int pgn); 
      bool decoder(int pgn, const BYTE DATA[8]);     
      void horodater();  
      void ajouterMesures(int pgn);
 
   private:
      std::string lieu;
      InterfacePeakNMEA2000 interfacePeakNMEA2000;
      Mesures mesures;
      double temperature;
      double pressionAtmospherique;
      double hygrometrie;
      double vitesseVent;
      double directionVent;
      time_t timestamp;
      ofstream ofile;
};

#endif
